import os
import boto3 as boto

class S3Client(object):
    def __init__(self, bucket_name):
        self.client = boto.resource('s3',
            aws_access_key_id = 'AKIAYU3L4PNIAG2VIUWY',
            aws_secret_access_key = '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')
        self.bucket_name = bucket_name

    def __create_bucket(self):
        """Create bucket in remote s3
        """

        self.client.create_bucket(Bucket=self.bucket_name)


    def __print_all_buckets(self):
        """Prints all bucket available in remote s3
        """

        for bucket in self.client.buckets.all():
            print(bucket.name)

    def view(self):
        """Prints all objects presents in current bucket
        """

        bucket_instance = self.client.Bucket(self.bucket_name)
        for obj in bucket_instance.objects.all():
            print(obj.key)

    def download(self, s3_path: str, dst_path: str):
        """Downloads file to local cache

        Args:
            s3_path (str): path to file in remote bucket
            dst_path (str): path of file in local location
        """

        bucket_instance = self.client.Bucket(self.bucket_name)
        bucket_instance.download_file(Key=s3_path, Filename=dst_path)
        print(f"File {s3_path} transferred to {dst_path}.")

    def upload(self, content: str, target: str):
        """Uploads a given UTF-8 content to remote target

        Args:
            content (str): string content to wite in remote bucket
            target (str): location in remote file to write content to
        """
        
        self.client.Object(self.bucket_name, target).put(Body=content)


import os
import re
import pandas as pd
from onpremgpusdk.data.__s3util__ import S3Client
from botocore import exceptions
from pathlib import Path


LOCAL_BUCKET_PATH = os.path.join(str(Path.home()), ".onpremgpusdk")

class DataInstance(S3Client):

    def __init__(self, bucket_name: str):
        super(DataInstance, self).__init__(bucket_name)

    def write_csv(self, df: pd.DataFrame, target: str):
        """Writes a dataframe to remote bucket and update data in local file for consistency

        Args:
            df (pd.DataFrame): Dataframe to write to remote bucket
            target (str): target to write file to in remote bucket
        """
        
        csv_content = df.to_csv()
        self.upload(csv_content, target)

        # Update local cache
        localFilePath = self.__create_path(target)
        with open(localFilePath, 'w+') as localCacheFile:
            localCacheFile.write(csv_content)

    def read_csv(self, path: str) -> pd.DataFrame:
        """Reads csv file to Data fram from cache or remote bucket

        Args:
            path (str): Path of file in remote bucket

        Returns:
            pd.DataFrame: Data frame of the csv to read 
        """

        filePath = self.__create_path(path)
        if(os.path.isfile(filePath)):
            print("Reading data from cache")
            return pd.read_csv(filePath)
        else:
            print(f"Fetching data from {self.bucket_name}")
            return self.__refresh_from_bucket(path, filePath)

    def __create_path(self, path: str) -> str:
        """ Creates local path from remote path 

        Args:
            path (str): Path of file in remote bucket

        Returns:
            str: Path of the file in local cache
        """

        filePath = os.path.join(LOCAL_BUCKET_PATH, self.bucket_name, path)
        if not os.path.exists(os.path.dirname(filePath)):
            # Create folders if doesn't exist
            os.makedirs(os.path.dirname(filePath), exist_ok=True)
        return filePath

    def __refresh_from_bucket(self, path: str, filePath: str) -> pd.DataFrame:
        """ Refreshes and returns content from remote bucket

        Args:
            path (str): Path of file in remote bucket
            filePath (str): Path of file in local machine

        Raises:
            e: is raised in case of unknown 

        Returns:
            pd.DataFrame: Data frame of the csv to read 
        """

        try:
            self.download(path, filePath)
            return pd.read_csv(filePath)
        except exceptions.ClientError as e:
            if e.response['Error']['Code'] == "404":
                print(f"File not found in remote bucket {self.bucket_name}")
            else:
                raise e